/*
 * Copyright (c) 2003, The Regents of the University of California, through
 * Lawrence Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy). All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * (2) Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * (3) Neither the name of the University of California, Lawrence Berkeley
 * National Laboratory, U.S. Dept. of Energy nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * You are under no obligation whatsoever to provide any bug fixes, patches, or
 * upgrades to the features, functionality or performance of the source code
 * ("Enhancements") to anyone; however, if you choose to make your Enhancements
 * available either publicly, or directly to Lawrence Berkeley National
 * Laboratory, without imposing a separate written license agreement for such
 * Enhancements, then you hereby grant the following license: a non-exclusive,
 * royalty-free perpetual license to install, use, modify, prepare derivative
 * works, incorporate into other computer software, distribute, and sublicense
 * such enhancements or derivative works thereof, in binary and source code
 * form.
 */
package nux.xom.tests;


import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.List;

import javax.xml.transform.TransformerException;

import nu.xom.Document;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;
import nu.xom.ValidityException;
import nux.xom.pool.BuilderPool;
import nux.xom.pool.DocumentURIResolver;
import nux.xom.pool.XOMUtil;
import nux.xom.xquery.XQuery;

/**
 * Demo that runs the W3C XQuery use cases (part of saxon download) 
 * against our XQuery implementation.
 * 
 * @author whoschek.AT.lbl.DOT.gov
 * @author $Author: hoschek3 $
 * @version $Revision: 1.4 $, $Date: 2004/12/22 17:19:32 $
 */
public final class XQueryUseCasesTest {

	private XQueryUseCasesTest() {}
	
	public static void main(String[] args) throws Exception {
		//File rootDir = new File("/Users/hoschek/unix/java/share/xml/saxonb-8.2/use-cases");
		File rootDir = new File("use-cases");
		String NUL = null;
		String[] includeDirs =  { "xmp", "tree", "seq", "r", "parts", "sgml"       , "string",   "ns" };
		String[] includeFiles = { NUL,   NUL,    NUL,   NUL,  NUL,    "report.xml" , "news.xml", "auction.xml"};
//		String[] includeDirs = { "ns" };
//		String[] includeFiles = { "auction.xml" };
		final List excludes = java.util.Arrays.asList( new String[] 
			{ "ns/addq2.xq", "xmp/addq4.xq", "xmp/addq5.xq", "xmp/addq6.xq", "xmp/addq7.xq", "r/q12temp.xq"} );
			//{ "ns/q1.xq", "ns/addq2.xq", "xmp/addq4.xq", "xmp/addq5.xq", "xmp/addq6.xq", "xmp/addq7.xq", "r/q12temp.xq"} );
		
		char sep = File.separatorChar;
		int done = 0;
		for (int i=0; i < includeDirs.length; i++) {
			File dir = new File(rootDir.getPath() + sep + includeDirs[i]);
			Document doc = null;
			if (includeFiles[i] != null) doc = BuilderPool.GLOBAL_POOL.getBuilder(false).build(
					new File(dir.getPath() + sep + includeFiles[i]));
			
			File[] files = dir.listFiles(
				new FilenameFilter() {
					public boolean accept(File dir, String name) {
						return name.endsWith(".xq") && !excludes.contains(dir.getName()+"/"+name);
					}
				}
			);
			
			for (int j=0; j < files.length; j++) {
				File file = files[j];
				System.out.println("\n##############################################");
				System.out.println("now running: " + file);
				//XQuery xquery = XQueryPool.GLOBAL_POOL.getXQuery(file);
				String query = XOMUtil.toString(new FileInputStream(file), null);
				XQuery xquery = new XQuery(query, file.toURI(), null, getResolver());
				Nodes nodes = xquery.execute(doc).toNodes();
				StringBuffer buf = new StringBuffer();
				for (int k=0; k < nodes.size(); k++) {
					Node node = nodes.get(k);
					//String str = node.toXML();
					String str = XOMUtil.toPrettyXML(node);
					buf.append(str);
				}
				System.out.println(buf);
				
				int k = file.getPath().lastIndexOf(".");
				File out = new File(file.getPath().substring(0, k) + ".out");
				if (out.exists()) {
					System.out.println("-----------EXPTECTED OUTPUT: ------------");
					System.out.println(new String(XOMUtil.toByteArray(new FileInputStream(out))));
				}
				else {
					System.out.println("---- NO EXPECTED OUTPUT AVAILABLE ------");
				}
				done++;

			}
		}
		
		System.out.println("\nDone with " + done + " total tests. Goodbye.");		
	}

	private static DocumentURIResolver getResolver() {
		//if (true) return null;
		return new DocumentURIResolver() {
			public Document resolve(String href, String baseURI) throws ValidityException, ParsingException, IOException, TransformerException {
				System.out.println("resolving href="+href+", baseURI="+baseURI);
				String systemID = new net.sf.saxon.StandardURIResolver().resolve(href, baseURI).getSystemId();
				System.out.println("resolving systemID="+systemID);
				return BuilderPool.GLOBAL_POOL.getBuilder(false).build(systemID);
			}
		};
	}

}

